<?php
// $Id: bubbletimer.drush.inc,v 1.1.2.5 2010/03/12 08:39:29 yorirou Exp $

define('BUBBLETIMER_PRINT_NODE_TITLE_MAXLEN', 32);
define('BUBBLETIMER_PATTERN_LENGTH', 96);

/**
 * Implementation of hook_drush_help().
 *
 * @param string $section
 * @return string
 */
function bubbletimer_drush_help($section) {
  switch($section) {
    case 'drush:bubbletimer get':
      return dt('Returns a timesheet from a specified day.');
  }
}

/**
 * Implementation of hook_drush_command().
 *
 * Available commands in the bubbletimer namespace:
 * - get
 * - mark
 * - unmark
 * - nodealias list
 * - nodealias get
 * - nodealias set
 * - nodealias del
 *
 * @todo examples
 * @return array
 */
function bubbletimer_drush_command() {
  $command = array();
  $mandatory_arguments = array(
    'user' => 'User ID or name.',
  );
  $marking_arguments = $mandatory_arguments + array(
    'node' => 'Node ID, title or alias.',
    'markstring' => 'TODO',
  );
  $nodealias_arguments = $mandatory_arguments + array(
    'node' => 'Node ID, title or alias.',
  );
  $day_option = array(
    '--day' => 'Overrides the default day (today).',
  );
  $node_option = array(
    '--nodes' => 'Overrides the bubbletimer node list. Format: nid1:nid2:nid3',
  );
  $options = $day_option + $node_option;

  $command['bubbletimer get'] = array(
    'callback' => '_bt_get',
    'description' => 'Returns a timesheet from a specified day.',
    'bootstrap' => DRUSH_BOOTSTRAP_DRUPAL_FULL,
    'arguments' => $mandatory_arguments,
    'options' => $options,
    'aliases' => array(
      'btget',
    ),
    'examples' => array(
      // TODO
    ),
  );

  $command['bubbletimer mark'] = array(
    'callback' => '_bt_mark',
    'description' => 'Marks a time slot.',
    'boostrap' => DRUSH_BOOTSTRAP_DRUPAL_FULL,
    'arguments' => $marking_arguments,
    'options' => $day_option,
    'aliases' => array(
      'btmark', 'btm',
    ),
    'examples' => array(
      // TODO
    ),
  );

  $command['bubbletimer unmark'] = array(
    'callback' => '_bt_unmark',
    'description' => 'Unmarks a time slot.',
    'bootstrap' => DRUSH_BOOTSTRAP_DRUPAL_FULL,
    'arguments' => $marking_arguments,
    'options' => $day_option,
    'aliases' => array(
      'btunmark', 'btum', 'btu',
    ),
    'examples' => array(
      // TODO
    ),
  );

  $command['bubbletimer nodealias list'] = array(
    'callback' => '_bt_nodealias_list',
    'description' => 'Lists bubbletimer node aliases.',
    'bootstrap' => DRUSH_BOOTSTRAP_DRUPAL_FULL,
    'arguments' => $mandatory_arguments,
    'aliases' => array(
      'btnl', 'bubbletimer nodealias'
    ),
    'examples' => array(
      // TODO
    ),
  );

  $command['bubbletimer nodealias del'] = array(
    'callback' => '_bt_nodealias_del',
    'description' => 'Deletes a bubbletimer node alias.',
    'bootstrap' => DRUSH_BOOTSTRAP_DRUPAL_FULL,
    'arguments' => $nodealias_arguments,
    'aliases' => array(
      'btnd',
    ),
    'examples' => array(
      // TODO
    ),
  );

  $command['bubbletimer nodealias set'] = array(
    'callback' => '_bt_nodealias_set',
    'description' => 'Sets a bubbletimer nodealias.',
    'bootstrap' => DRUSH_BOOTSTRAP_DRUPAL_FULL,
    'arguments' => $nodealias_arguments + array(
      'alias' => 'The new alias.',
    ),
    'aliases' => array(
      'btns',
    ),
    'examples' => array(
      // TODO
    ),
  );

  return $command;
}

/**
 * Shortcut function to access the commonly used options.
 *
 * @staticvar array $cache
 * @return array
 */
function _bt_get_options() {
  static $cache = NULL;
  if($cache === NULL) {
    $cache = array(
      'day' => drush_get_option('day', date('Y-m-d')),
      'nodes' => drush_get_option('nodes', NULL),
    );
  }
  return $cache;
}

/**
 * Loads the bubbletimer includes,
 * and executes the hook_init().
 */
function _bt_load_rt() {
  module_load_include('module', 'bubbletimer');
  bubbletimer_init();
}

/**
 * Tries to guess the current user.
 *
 * It can match by uid and user name.
 *
 * @param mixed $u
 * @param boolean $login
 *  If this true, then the user will log in. @see _bt_user_login().
 * @return stdClass
 *  The user object. It will be FALSE if the user is not found.
 */
function _bt_get_user($u, $login = FALSE) {
  $u = user_load(is_numeric($u) ? $u : array('name' => $u));
  if($u === FALSE || $u->uid == 0) {
    return drush_set_error('DRUSH_BUBBLETIMER_INVALID_USER', dt('The specified user does not exists.'));
  }
  if($login) _bt_user_login($u);
  return $u;
}

/**
 * Replaces the global user object.
 *
 * @global stdClass $user
 * @param stdClass $u
 */
function _bt_user_login($u) {
  global $user;
  $user = $u;
}

/**
 * Tries to guess node(s) based on the $node_string.
 *
 * It breaks the $node_string into fragments, and tries to match
 * nodes to them. On the first round, it checks if the fragment is
 * numeric, and assumes it as a node id. The second run is the title
 * matching, the third is the alias matching unless the aliases are
 * disabled by the $use_aliases parameter.
 *
 * @todo add some error handling
 * @param stdClass $user
 * @param string $node_string
 *  node ids/titles/aliases separated by :
 * @param boolean $use_aliases
 * @return array
 *  Node IDs and titles as an associative array.
 */
function _bt_get_nodes($user, $node_string = NULL, $use_aliases = TRUE) {
  module_load_include('inc', 'bubbletimer', 'nodealias');
  $nodes = $node_string === NULL ? drush_get_option('nodes', NULL) : $node_string;
  if($nodes === NULL) {
    $nodes = bubbletimer_get_jobs_assigned($user);
  } else {
    $node_array = explode(':', $nodes);
    $nodes = array();
    foreach($node_array as $n) {
      if(is_numeric($n)) { // load by nid
        $no = node_load($n);
        $nodes[$n] = $no->title;
      } else { // load by title
        $node = node_load(array('title' => $n));
        if($node) {
          $nodes[$node->nid] = $node->title;
        } else {
          if($use_aliases) { // load by alias
            $alias_nid = bubbletimer_nodealias_load_by_alias($user, $n);
            $no = node_load($alias_nid);
            if($alias_nid && $no && is_object($no)) {
              $nodes[$alias_nid] = $no->title;
            }
          }
        }
      }
    }
  }
  return $nodes;
}

/**
 * Renders a header for printing.
 *
 * This function is a helper function for _bt_get().
 *
 * @see _bt_get()
 * @return string
 */
function _bt_pattern_header() {
  $header = str_pad('| ', BUBBLETIMER_PRINT_NODE_TITLE_MAXLEN + 5, ' ', STR_PAD_LEFT);

  for($i = 0; $i < 24; ++$i) {
    $header .= '  ' . ($i < 10 ? "0$i" : $i) . ' ';
  }

  $header .= ' |';

  return $header;
}

/**
 * Formats a title and a pattern for printing.
 *
 * This is a helper function for _bt_get().
 *
 * @see _bt_get()
 * @param string $title
 * @param string $pattern
 *  Bubbletimer "bitpattern". It's length is
 *  defined in the BUBBLETIMER_PATTERN_LENGTH
 *  constant.
 * @return string
 */
function _bt_format_pattern($title, $pattern) {
  $pattern = str_replace('1', '#', $pattern);
  $pattern = str_replace('0', '-', $pattern);
  $orig_pattern = $pattern;
  $pattern = '';
  for($i = 0; $i < BUBBLETIMER_PATTERN_LENGTH; ++$i) {
    $pattern .= (($i % 4 == 0) ? ' ' : '') . $orig_pattern{$i};
  }
  unset($orig_pattern);
  $title = trim($title);
  $title_len = strlen($title);
  if($title_len != BUBBLETIMER_PRINT_NODE_TITLE_MAXLEN) {
    $title = $title_len > BUBBLETIMER_PRINT_NODE_TITLE_MAXLEN ?
            substr($title, 0, BUBBLETIMER_PRINT_NODE_TITLE_MAXLEN-3).'...':
            str_pad($title, BUBBLETIMER_PRINT_NODE_TITLE_MAXLEN, ' ');
  }
  return sprintf('| %s | %s |', $title, $pattern);
}

/**
 * Callback function for "bubbletimer get".
 *
 * @param string $user
 *  User identification string.
 *  It can be a user id or name.
 */
function _bt_get($user) {
  $args = func_get_args();
  array_shift($args);
  $user = _bt_get_user($user, TRUE);
  if(!$user) return FALSE;
  _bt_load_rt();

  $nodes = _bt_get_nodes($user);

  if(count($nodes)) {
    drush_print(_bt_pattern_header());
  }

  foreach($nodes as $nid => $title) {
    $pattern = _bt_get_job_pattern($user->uid, $nid);
    drush_print(_bt_format_pattern($title, $pattern));
  }
}

/**
 * Returns a job pattern based on the --day option.
 *
 * @param int $uid
 * @param int $nid
 * @return string
 */
function _bt_get_job_pattern($uid, $nid) {
  $pattern = bubbletimer_get_job_pattern($uid, $nid, drush_get_option('day', date('Y-m-d')));
  if($pattern === NULL) {
    // If there is no entry in the database,
    // bubbletimer_get_job_pattern() will
    // return NULL, so we initialize an
    // empty pattern.
    $pattern = str_repeat('0', BUBBLETIMER_PATTERN_LENGTH);
  }
  return $pattern;
}

/**
 * Helper function for marking and unmarking functions.
 *
 * This function uses the strategy pattern.
 *
 * @param string $user
 * @param string $nodes
 * @param string $markstring
 * @param callback $maskop_callback
 * @return bool
 */
function _bt_mark_common(&$user, &$nodes, &$markstring, $maskop_callback) {
  if(!_bt_mark_init($user, $nodes)) return FALSE;
  $mask = _bt_markstring_to_mask($markstring);
  foreach($nodes as $nid => $title) {
    $pattern = _bt_get_job_pattern($user->uid, $nid);
    for($i = 0; $i < BUBBLETIMER_PATTERN_LENGTH; ++$i) {
      $pattern{$i} = $maskop_callback($patter{$i}, $mask{$i});
    }
    var_dump($pattern, $nid, drush_get_option('day', date('Y-m-d')));
    db_query('UPDATE {bubbletimer_activity} SET activity = \'%s\' WHERE nid = %d AND uid = %d AND date = \'%s\'',
            $pattern, $nid, $user->uid, drush_get_option('day', date('Y-m-d')));
    if(!db_affected_rows()) {
      db_query('INSERT INTO bubbletimer_activity("date", nid, uid, activity) VALUES(\'%s\', %d, %d, \'%s\')',
              drush_get_option('day', date('Y-m-d')), $nid, $user->uid, $pattern);
    }
  }
  return TRUE;
}

/**
 * Callback function for the "bubbletimer mark" action.
 *
 * @param string $user
 * @param string $nodes
 * @param string $markstring
 * @return bool
 */
function _bt_mark($user, $nodes, $markstring) {
  return _bt_mark_common($user, $nodes, $markstring,
          create_function('$pattern, $mask', 'return (int)(bool)($pattern != 0 || $mask != 0);'));
}

/**
 * Callback function for the "bubbletimer unmark" action.
 *
 * @param string $user
 * @param string $nodes
 * @param string $markstring
 * @return bool
 */
function _bt_unmark($user, $nodes, $markstring) {
  return _bt_mark_common($user, $nodes, $markstring,
          create_function('$pattern, $mask', 'return (int)(bool)($mask ? 0 : $pattern);'));
}

/**
 * Converts a markstring into a mask.
 *
 * @param string $markstring
 * @return string
 */
function _bt_markstring_to_mask($markstring) {
  $mask = str_repeat('0', BUBBLETIMER_PATTERN_LENGTH);
  foreach(explode(':', $markstring) as $fragment) {
    $interval = explode('-', $fragment);
    $start = explode('.', $interval[0]);
    $stop = $start;
    if(isset($interval[1])) {
      $stop = explode('.', $interval[1]);
      if($stop[0] == '') {
        $stop[0] = $start[0];
      }
    }
    do {
      if(isset($start[1])) {
        $mask{$start[0]*4+$start[1]-1} = '1';
      } else {
        for($i = 0; $i < 4; ++$i) {
          $mask{$start[0]*4+$i} = '1';
        }
      }
      if($start[1] == 4) {
        $start[1] = 0;
        $start[0]++;
      } else {
        $start[1]++;
      }
    } while(!($start[0] == $stop[0] && $start[1] == $stop[1]) && $start[0] < 23);
  }
  return substr($mask, 0, BUBBLETIMER_PATTERN_LENGTH);
}

/**
 * Initializes the context for the (un)mark functions.
 *
 * @param string $user
 * @param string $nodes
 * @return bool
 */
function _bt_mark_init(&$user, &$nodes) {
  $user = _bt_get_user($user, TRUE);
  if(!$user) return FALSE;
  _bt_load_rt();
  $nodes = _bt_get_nodes($user, $nodes);
  return TRUE;
}

/**
 * Tries to initialize the user variable
 * for the nodealias commands.
 *
 * @param string $user
 *  User identification string.
 *  It can be the uid or name.
 * @return boolean
 */
function _bt_nodealias_init(&$user) {
  $user = _bt_get_user($user, TRUE);
  if(!$user) return FALSE;
  _bt_load_rt();
  module_load_include('inc', 'bubbletimer', 'nodealias');
  return TRUE;
}

/**
 * Returns the node id based on user and node string.
 *
 * This is a helper function for the nodealias commands.
 *
 * @param stdClass $user
 * @param string $node
 * @param boolean $use_aliases
 * @return int
 */
function _bt_nodealias_get_nid($user, $node, $use_aliases) {
  $node = _bt_get_nodes($user, $node, $use_aliases);
  if(!is_array($node) || !count($node)) {
    return drush_set_error('DRUSH_BUBBLETIMER_INVALID_NODE', 'The specified node does not exists.');
  }
  $nid = array_keys($node);
  $nid = array_shift($nid);

  return $nid;
}

/**
 * Generates a header for the "bubbletimer nodealias list" command.
 *
 * @param int $max_nid
 * @param int $max_title
 * @param int $max_alias
 * @return string
 */
function _bt_nodealias_list_header($max_nid, $max_title, $max_alias) {
  $output =
    '| ' . str_pad(dt('Node ID'), $max_nid) . ' | '
    . str_pad(dt('Title'), $max_title) . ' | '
    . str_pad(dt('Alias'), $max_alias) . ' |';
  $output .= "\n" . str_repeat('-', strlen($output));
  return $output;
}

/**
 * Formatting function for the "bubbletimer nodealias list" command.
 *
 * @param int $nid
 * @param string $title
 * @param string $alias
 * @param int $max_nid
 * @param int $max_title
 * @param int $max_alias
 * @return string
 */
function _bt_nodealias_list_item($nid, $title, $alias, $max_nid, $max_title, $max_alias) {
  return '| ' . str_pad($nid, $max_nid) . ' | ' . str_pad($title, $max_title) . ' | ' . str_pad($alias, $max_alias) . ' |';
}

/**
 * Callback for the "bubbletimer nodealias list" command.
 *
 * @param string $user
 */
function _bt_nodealias_list($user) {
  if(!_bt_nodealias_init($user)) return FALSE;
  $rows = bubbletimer_nodealias_list($user->uid);
  $max_nid = strlen(dt('Node ID'));
  $max_title = strlen(dt('Title'));
  $max_alias = strlen(dt('Alias'));
  foreach($rows as $row) {
    if(strlen($row['nid']) > $max_nid) {
      $max_nid = strlen($row['nid']);
    }
    if(strlen($row['title']) > $max_title) {
      $max_title = strlen($row['title']);
    }
    if(strlen($row['alias']) > $max_alias) {
      $max_alias = strlen($row['alias']);
    }
  }

  drush_print(_bt_nodealias_list_header($max_nid, $max_title, $max_alias));
  foreach($rows as $row) {
    drush_print(_bt_nodealias_list_item($row['nid'], $row['title'], $row['alias'], $max_nid, $max_title, $max_alias));
  }
}

/**
 * Callback function for "bubbletimer nodealias del" command.
 *
 * @param string $user
 * @param string $node
 */
function _bt_nodealias_del($user, $node) {
  if(!_bt_nodealias_init($user)) return FALSE;

  $nid = _bt_nodealias_get_nid($user, $node, TRUE);

  bubbletimer_nodealias_delete($user->uid, $nid);
}

/**
 * Callback function for "bubbletimer nodealias set" command.
 *
 * @param string $user
 * @param string $node
 * @param string $alias
 */
function _bt_nodealias_set($user, $node, $alias) {
  if(!_bt_nodealias_init($user)) return FALSE;

  $nid = _bt_nodealias_get_nid($user, $node, FALSE);
  if($nid === FALSE) return FALSE;

  bubbletimer_nodealias_save($user->uid, $nid, $alias);
}