<?php
// $Id: bubbletimer.page.inc,v 1.1.2.3.2.12 2010/03/18 08:06:22 yorirou Exp $


/**
 * @file
 * Bubbletimer main page
 */

function bubbletimer_page_main() {
  drupal_goto('bubbletimer/nojs');
}

/**
 * Bubbletimer interface
 * Render
 * 	- title
 * 	- date selector form part
 * 	- activity form
 * @return string
 */
function bubbletimer_page($js = FALSE, $nid = NULL) {
  bubbletimer_include_js_css();

  $date_form = drupal_get_form('bubbletimer_date_selector_form');

  $user = bubbletimer_get_user();
  $date = bubbletimer_get_date();
  $timestamp = strtotime(join('-', $date));

  drupal_set_title(t('Bubbletimer - @username\'s daily basis', array('@username' => $user->name)));
  
  $nodes_enabled = array();
  if($nid === NULL) {
    $nodes_enabled = bubbletimer_get_enabled_jobs(bubbletimer_get_user());
  } else {
    $nodes_enabled = bubbletimer_ahah_check_nid($nid);
  }
  $node_form = $js ? '' : drupal_get_form('bubbletimer_node_add_form', count($nodes_enabled) == 0);
  if (!empty($nodes_enabled)) {
    $bubbletimer_form = drupal_get_form('bubbletimer_activityform', $nodes_enabled, !$js);
  } else {
    $bubbletimer_form = '';
  }

  $output = theme('bubbletimer_page', $date_form, $node_form, $bubbletimer_form);
  if($js) {
    ctools_include('ajax');

    $commands = array();
    if($nid !== NULL) {
      $commands []= ctools_ajax_command_after('#bubbletimer-ahah-link', theme_bubbletimer_ahah_hide_link($nid));
    }
    // TODO do something with the following 4 commands
    $commands []= ctools_ajax_command_remove('#bubbletimer');
    $commands []= ctools_ajax_command_before('#bubbletimer-remove-link', $output);
    $commands []= ctools_ajax_command_remove('#bubbletimer-ahah-link');
    $commands []= ctools_ajax_command_reinit_date_selector_form();
    $commands []= ctools_ajax_command_activate_hide_link(theme_bubbletimer_ahah_show_link($nid));
    $commands []= ctools_ajax_command_reinit();

    ctools_ajax_render($commands);
  } else {
    return $output;
  }
}


/**
 * Definition of activity form
 * @return array
 */
function bubbletimer_activityform($form_state, $nids, $quickchart = TRUE) {
  $user = bubbletimer_get_user();
  //$nids = bubbletimer_get_enabled_jobs($user);
  $timestamp_original = bubbletimer_get_timestamp();

  $form['bt']['#quickchart'] = $quickchart;

  $form['bt']['#theme'] = 'bt_form';

  $form['bt']['bt_wrapper'] = array(
    '#theme' => 'bt_wrapper',
  );

  // Checkboxes and titles / node / step
  foreach ((array)$nids as $nid => $node_title) {
    $timestamp = $timestamp_original;
    $form['bt']['bt_wrapper']['title'][$nid] = array(
      '#value' => '<a class="job_node" nid="'. $nid .'" href="'. url('node/'.  $nid, array('absolute' => TRUE)) .'">'.
                  check_plain((strlen($node_title) > BT_TITLE_LENGTH ? substr($node_title, 0, BT_TITLE_LENGTH) .'...' : $node_title)) .
                  ' </a><small>, <span id="bt_node_percent_'. $nid .'"></span>h</small>',
      '#nid' => $nid,
    );
    $nid_pattern = bubbletimer_get_job_pattern($user->uid, $nid, date('Y-m-d', bubbletimer_get_timestamp()));
    for ($i = 0; $i < BT_STAGE_COUNT; $i++) {
      $form['bt']['bt_wrapper']['slot'][$nid][$timestamp] = array(
        '#type' => 'bubbletimer_slot',
        '#default_value' => substr($nid_pattern, $i, 1) == '1' ? 1 : 0,
      );
      $timestamp += BT_SECONDS_INTERVAL;
    }
  }

  // Submit button
  $form['bt']['bt_submit'] = array(
    '#type' => 'submit',
    '#value' => 'Save changes',
    '#suffix' => '
    	<input type="button" onclick="bubbletimer_history_backward()" value="'.t('Undo').'" />
    	<input type="button" onclick="bubbletimer_history_forward()" value="'.t('Redo').'" />
    ',
  );

  return $form;
}


/**
 * Define date selector form
 * @param array $form_state information about form
 * @return string
 */
function bubbletimer_date_selector_form($form_state) {
  $f = array();

  $f['bt_date'] = array(
    '#title' => t('Go to'),
    '#type' => 'date_popup', // TODO "module_exists('date_popup') ? 'date_popup' : 'textfield'" if needed
    '#date_format' => 'm/d/Y',
    '#default_value' => format_date(time(), "custom", "Y-m-d"),
  );

  $f['bt_submit'] = array(
    '#type' => 'submit',
    '#value' => t('Update'),
  );

  $f['bt_date_last'] = array(
    '#type' => 'button',
    '#value' => t('Last edited day'),
  );

  $f['bt_date_now'] = array(
    '#type' => 'button',
    '#value' => t('Today'),
  );

  return $f;
}


/**
 * Assign or unassign a certain node to a user -> to get it into the timesheet list
 * @param integer $nid
 * @param boolean $is_assign
 */
function bubbletimer_assign_node_to_user($nid, $is_assign, $silent = FALSE) {
  $user = bubbletimer_get_user();
  
  // Sanitizing
  $nid = intval($nid);
  $uid = intval($user->uid);

  $result = db_fetch_object(db_query(
    'SELECT count(*) AS count FROM {bubbletimer_node_user} WHERE uid = %d AND nid = %d;',
    $uid,
    $nid
  ));
  if ($is_assign && $result->count == 0) {
    $result = db_query('INSERT INTO {bubbletimer_node_user} (uid, nid) VALUES (%d, %d);', $uid, $nid);
    if(!$silent) drupal_set_message(t('This node will be in your timesheet list'));
  }
  else if (!$is_assign && $result->count > 0) {
    $result = db_query('DELETE FROM {bubbletimer_node_user} WHERE uid = %d AND nid = %d;', $uid, $nid);
    if(!$silent) drupal_set_message(t('You deleted this node from your timesheet'));
  }
}


/**
 * Implementation of hook_submit()
 * Saving selected day in session
 * @param array $form
 * @param arary $form_state
 */
function bubbletimer_date_selector_form_submit($form, $form_state) {
  $bt_date = $form_state['values']['bt_date'];
  $datearr = explode('-', date('Y-m-d', (!empty($bt_date) ? strtotime($bt_date) : time())));
  bubbletimer_set_date(array('year' => $datearr[0], 'month' => $datearr[1], 'day' => $datearr[2]));
  //drupal_set_message(t('Day is changed to: ') . date('Y. m. d. (l)', bubbletimer_get_timestamp(BT_TIMESTAMP_ONLY_DAY)));
}


/**
 * Implmentation of hook_submit()
 * Saving daily activities into database
 * @param array $form
 * @param array $form_state
 */
function bubbletimer_activityform_submit($form, $form_state) {
  $user = bubbletimer_get_user();

  foreach ((array)$form_state['clicked_button']['#post']['slot'] as $nid => $form_row) {
    // @TODO use #tree instead of this hackish access
    $activity = '';
    foreach ((array)$form_row as $timestamp => $value) {
      $activity .= $value == '1' ? '1' : '0'; // Sanitizing
    }
    $result = db_fetch_object(db_query('
      SELECT count(*) as count FROM {bubbletimer_activity} WHERE date = \'%s\' AND nid = %d AND uid = %d;
      ', join('-', bubbletimer_get_date()), intval($nid), intval($user->uid)));
    if ($result->count > 0) {
      db_query('
        UPDATE {bubbletimer_activity} SET activity = \'%s\' WHERE date = \'%s\' AND nid = %d AND uid = %d;
        ', $activity, join('-', bubbletimer_get_date()), intval($nid), intval($user->uid));
    } else {
      db_query('
        INSERT INTO {bubbletimer_activity} (date, nid, uid, activity) VALUES (\'%s\', %d, %d, \'%s\');
        ', join('-', bubbletimer_get_date()), intval($nid), intval($user->uid), $activity);
    }

    // If node updating is enabled.
    // Ex.: Computed field needs node update to recalculate data.
    if (variable_get('bubbletimer_node_refresh', '0') == '1') {
      $node = node_load($nid);
      node_save($node);
    }
  }

  //drupal_set_message(t('You saved your timesheet succesfully'));
}


/**
 * Implementation of theme_hook()
 * Just for adding a submit right next to the date selectors
 * @param array $element
 * @return string
 */
function theme_bubbletimer_date_field($element) {
  return drupal_render($element) .'<input type="submit" value="Go" />';
}


/**
 * Implementation of hook_form_alter()
 * Setup forms:
 * 	- Set activity in the form (from datas in bubbletimer table)
 * 	- Set the current date on date selector form
 * @param array $form
 * @param array $form_state
 * @param string $form_id
 */
function bubbletimer_form_alter(&$form, &$form_state, $form_id) {
  if ($form_id == 'bubbletimer_date_selector_form' && empty($_POST)) {
    $date_array = array_map(create_function('$item', 'return (int)$item;'), bubbletimer_get_date());
    //$form['bt_dayselector_fieldset']['bt_date']['#value'] = $date_array;
    $form['bt_date']['#value'] = format_date(strtotime("{$date_array['year']}-{$date_array['month']}-{$date_array['day']}"), 'custom', 'm/d/Y');
  }
  else if ($form_id == 'bubbletimer_node_assign_form' && empty($_POST)) {
    $user = bubbletimer_get_user();
    $form['bubbletimer_node_assign_fieldset']['bt_assign_checkbox']['#value'] =
      bubbletimer_is_node_assign_to_user($user->uid, $form['bubbletimer_node_assign_fieldset']['bt_node_nid']['#value']);
  }
}


/**
 * Form for adding nodes to bt on bubbletimer's page
 * @param array $form_state
 * @param boolean $enable_nodes_empty - if no nodes enables, it gets not collapsed
 * @return array
 */
function bubbletimer_node_add_form($form_state, $enable_nodes_empty) {
  $nodes_available = bubbletimer_get_jobs_available();
  $node_types = node_get_types('types');
  foreach ((array)$node_types as $node_type) {
    $_node_type_info[$node_type->type] = $node_type->name;
  }
  foreach ((array)$nodes_available as $node_type => $nodes) {
    $node_types_available[$node_type] = $_node_type_info[$node_type];
  }

  $form['bt_nodeadd_fieldset'] = array(
    '#type' => 'fieldset',
    '#collapsible' => TRUE,
    '#collapsed' => !$enable_nodes_empty,
    '#title' => t('Add new node to the timesheet'),
  );
  if (!empty($node_types_available)) {
    $form['bt_nodeadd_fieldset']['bt_node_type'] = array(
      '#type' => 'select',
      '#title' => t('Node type'),
      '#options' => $node_types_available,
    );
  } else {
    $form['bt_nodeadd_fieldset']['bt_info'] = array(
      '#type' => 'item',
      '#title' => t('No conent types available for Bubbletimer. Set on the <a href="@link">Bubbletimer settings</a> page.',
              array('@link' => url('admin/settings/bubbletimer/general'))),
    );
  }
  foreach ((array)$nodes_available as $node_type => $nodes) {
    $form['bt_nodeadd_fieldset']['bt_addnode_select_'.$node_type] = array(
      '#type' => 'select',
      '#title' => $node_types_available[$node_type],
      '#options' => $nodes,
      '#attributes' => array(
        'class' => 'bubbletimer_node_select',
        'node_type' => $node_type,
      ),
    );
  }
  $form['bt_nodeadd_fieldset']['bt_addnode_submit'] = array(
    '#type' => 'submit',
    '#value' => t('Add'),
  );

  return $form;
}


/**
 * Submit handler for node-adding form
 *
 * @param array $form
 * @param array $form_state
 */
function bubbletimer_node_add_form_submit($form, $form_state) {
  bubbletimer_assign_node_to_user(intval($form_state['values']['bt_addnode_select_'.$form_state['values']['bt_node_type']]), TRUE);
  drupal_goto(BT_PATH);
}


function theme_bt_wrapper($form) {
  global $base_path;
  $path = $base_path.drupal_get_path('module', 'bubbletimer');

  $nodes_assigned = bubbletimer_get_jobs_assigned(bubbletimer_get_user());
  $nodes_assigned_keys = array_keys($nodes_assigned);

  $table_rows = array();
  $row = array();
  $row[] = array(
    'data' => '<span class="wait bt_chart_updater"></span>Projects <small>(<span class="bt_sum_hour"></span>h a day)</small>',
    'class' => 'bt_td_header',
  );

  $form_keys = array_keys($form['slot']);
  $nids = array_filter($form_keys, create_function('$item', 'return is_numeric($item);'));

  // Left arrow
  $row[] = array(
    'data' => '<img src="'.$path.'/images/step_left.gif" />',
    'rowspan' => count($nids) + 1,
    'id' => 'bt_table_step_left',
  );

  for ($hour = BT_START_HOUR; $hour < BT_END_HOUR; $hour++) {
    $row[] = array(
      'data' => $hour.'h',
      'colspan' => BT_SLOTS_PER_HOUR,
      'hour' => $hour,
      'class' => 'bt_td_header bt_hour_td '.($hour & 1 ? 'even' : 'odd'),
      'style' => $hour < 8 || $hour >= 18 ? 'display:none;' : '',
    );
  }

  // Right arrow
  $row[] = array(
    'data' => '<img src="'.$path.'/images/step_right.gif" />',
    'rowspan' => count($nids) + 1,
    'id' => 'bt_table_step_right',
  );
  $table_rows[] = $row;

  foreach ((array)$form['slot'] as $nid => $form_row) {
    if (!is_array($form_row) || !is_numeric($nid)) continue;
    $row = array();

    $title = array_shift($form['title']);
    //var_dump($title);
    $assign_link = in_array($nid, $nodes_assigned_keys) ?
      '<a class="bubbletimer_node_remove" href="'.url(BT_URL_NODE_REMOVE.'/'.$nid).'">&nbsp;</a>' :
      '<a class="bubbletimer_node_add"    href="'.url(BT_URL_NODE_ADD.'/'.$nid).'">&nbsp;</a>';
    $row[] = array(
      'data' => $assign_link.drupal_render($title),
      'class' => 'bt_title_td',
      'nid' => $nid,
    );

    $hour = BT_START_HOUR;
    foreach ((array)$form_row as $timestamp => $element) {
      //var_dump($element); exit;
      if (!is_numeric($timestamp)) continue;
      //var_dump($timestamp);
      $row[] = array(
        'data' => drupal_render($element),
        'hour' => floor($hour),
        'class' => 'bt_slot_td '.(floor($hour) & 1 ? 'even' : 'odd'),
        'valign' => 'bottom',
        'nid' => $nid,
        'style' => $hour < 8 || $hour >= 18 ? 'display:none;' : '',
      );
      $hour += 1 / BT_SLOTS_PER_HOUR;
    }
    $table_rows[] = $row;
  }

  $output  = theme_table(NULL, $table_rows, array(
    'id' => 'bt_table',
  ));
  //$output .= drupal_render($form);

  return $output;
}


/**
 * Node assign form.
 * @param array $form_original
 * @param int $nid
 * @param string $action
 * @return array
 */
function bubbletimer_assign_node_form($form_original, $nid, $action = BT_NODE_REMOVE) {
  drupal_set_title(t('Bubbletimer - @action node', array('@action' => $action == BT_NODE_ADD ? t('add') : t('remove'))));
  $node = node_load($nid);

 return array(
    'text' => array(
      '#type' => 'item',
      '#title' => t('Do you want to @action the node: @title?', array(
        '@action' => $action == BT_NODE_ADD ? t('add') : t('remove'),
        '@title' => $node->title,
      )),
    ),
    'nid' => array(
      '#type' => 'hidden',
      '#value' => (int)$nid,
    ),
    'action' => array(
      '#type' => 'hidden',
      '#value' => $action,
    ),
    'submit' => array(
      '#type' => 'submit',
      '#value' => $action == BT_NODE_ADD ? t('Add node') : t('Remove node'),
    ),
  );
}


/**
 * Submission for node assign form.
 * @param array $form
 * @param array $form_state
 */
function bubbletimer_assign_node_form_submit($form, $form_state) {
  bubbletimer_assign_node_to_user($form_state['values']['nid'], $form_state['values']['action'] == BT_NODE_ADD);
  drupal_goto('bubbletimer');
}

function bubbletimer_ahah_check_nid($nid) {
  global $user;
  $node = node_load($nid);
  if($node === FALSE || !node_access('view', $node)) return array();
  if(!db_result(db_query('SELECT COUNT(*) FROM {bubbletimer_node_user} WHERE uid = %d AND nid = %d', $user->uid, $node->nid))) return array();
  return array($node->nid => $node->title);
}

function bubbletimer_add_node($js, $node) {
  bubbletimer_assign_node_to_user($node->nid, TRUE, TRUE);
  ctools_include('ajax');
  $commands = array();
  $commands []= ctools_ajax_command_before("#bubbletimer-add-link", bubbletimer_theme_wrapper(theme_bubbletimer_ahah_show_link($node->nid)));
  $commands []= ctools_ajax_command_before("#bubbletimer-add-link", bubbletimer_theme_wrapper(theme_bubbletimer_remove_link($node->nid)));
  $commands []= ctools_ajax_command_remove("#bubbletimer-add-link");
  bubbletimer_crud_redirect_handler($js, $node, $commands);
  return NULL;
}

function bubbletimer_del_node($js, $node) {
  bubbletimer_assign_node_to_user($node->nid, FALSE, TRUE);
  ctools_include('ajax');
  $link = bubbletimer_theme_wrapper(theme_bubbletimer_add_link($node->nid));
  $commands = array();
  $commands []= ctools_ajax_command_before("#bubbletimer-ahah-link", $link);
  $commands []= ctools_ajax_command_before("#bubbletimer-ahah-hide-link", $link);
  $commands []= ctools_ajax_command_remove("#bubbletimer-ahah-link");
  $commands []= ctools_ajax_command_remove("#bubbletimer-ahah-hide-link");
  $commands []= ctools_ajax_command_remove("#bubbletimer");
  $commands []= ctools_ajax_command_remove("#bubbletimer-remove-link");
  bubbletimer_crud_redirect_handler($js, $node, $commands);
  return NULL;
}

function bubbletimer_crud_redirect_handler($js, $node, $commands) {
  if($js) {
    $commands []= ctools_ajax_command_reinit();
    ctools_ajax_render($commands);
  } else {
    drupal_goto('node/'.$node->nid);
  }
}